@@ -10,3 +10,5 @@ admin.site.register(models.CloudStackOffering, CloudStackServiceofferingAdmin)
 admin.site.register(models.CloudStackBundle, CloudStackBundleAdmin)
 admin.site.register(models.CloudStackRegion, CloudStackRegionAdmin)
 admin.site.register(models.CloudStackPack, CloudStackPackAdmin)
+admin.site.register(models.BundleGroup)
+admin.site.register(models.OfferingGroup)
