@@ -261,6 +261,40 @@ class CloudStackProvider(object):
             LOG.warning("We could not retrieve the vm networkid %s" % e)
             return None
 
+    def get_vm_zone_id(self, vm_id, project_id, affinity_group_id=None):
+        try:
+            LOG.info("Listing zone id for vm (id: %s)" % (vm_id))
+
+            request = {'projectid': project_id, 'id': vm_id}
+
+            if affinity_group_id:
+                request['affinitygroupids'] = affinity_group_id
+                del request['projectid']
+
+            response = self.api.listVirtualMachines('GET', request)
+
+            return response['virtualmachine'][0]['zoneid']
+        except Exception as e:
+            LOG.warning("We could not retrieve the vm zoneid %s" % e)
+            return None
+
+    def get_vm_template_id(self, vm_id, project_id, affinity_group_id=None):
+        try:
+            LOG.info("Listing template id for vm (id: %s)" % (vm_id))
+
+            request = {'projectid': project_id, 'id': vm_id}
+
+            if affinity_group_id:
+                request['affinitygroupids'] = affinity_group_id
+                del request['projectid']
+
+            response = self.api.listVirtualMachines('GET', request)
+
+            return response['virtualmachine'][0]['templateid']
+        except Exception as e:
+            LOG.warning("We could not retrieve the vm templateid %s" % e)
+            return None
+
     def get_vm_ostype(self, vm_id, project_id, affinity_group_id=None):
         try:
             LOG.info("Listing ostype for vm (id: %s)" % (vm_id))
@@ -413,3 +447,35 @@ class CloudStackProvider(object):
             LOG.warning(
                 "We could not remove networkapi equipment because %s" % e)
             return None
+
+    def reinstall_new_template(self, vm_id, template_id):
+        try:
+            LOG.info(
+                "Reinstall Virtualmachine (id: {}) template to {}".format(
+                    vm_id, template_id
+                )
+            )
+            response = self.api.restoreVirtualMachine(
+                'POST', {'virtualmachineid': vm_id, 'templateid': template_id}
+            )
+
+            if 'jobid' in response:
+                query_async = self.query_async_job(jobid=response['jobid'])
+                if query_async :
+                    LOG.info("VirtualMachine reinstalled!")
+                    return True
+                else:
+                    return False
+            else:
+                LOG.warning(
+                    "Something occurred on cloudstack: {}, {}".format(
+                        response['errorcode'], response['errortext']
+                    )
+                )
+                return
+
+        except Exception as e:
+            LOG.warning(
+                "We could not reinstall the virtualmachine because {}".format(e)
+            )
+            return
